﻿using Infragistics.Win;
using Infragistics.Win.UltraWinEditors;
using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ButtonsRight_Vertical
{
    class Vertical_ButtonsRight_CreationFilter : IUIElementCreationFilter
    {
        void IUIElementCreationFilter.AfterCreateChildElements(UIElement parent)
        {
            // Watch for EditorWithTextUIElement. This is the element that contains
            // the button elements and the text element.
            var editorWithTextUIElement = parent as EditorWithTextUIElement;            
            if (null != editorWithTextUIElement)
            {
                // Get the editor and the EditorButtons. There are two reasons why we do this, rather
                // than just getting the button UIElements from the parent. 
                // First, the elements in the collection are in reverse order, and presumably
                // we want the first button to be on top. 
                // Second, using the ButtonsRight collection on the editor allows us to differentiate
                // between ButtonsRight and ButtonsLeft - the UIElements don't have that information. 
                var editorWithText = editorWithTextUIElement.Editor as EditorWithText;
                var buttonsRight = editorWithText.ButtonsRight;
                int buttonCount = buttonsRight.Count;

                // If there are no ButtonsRight, do nothing. 
                if (buttonCount == 0)
                    return;

                // Create a list of the ButtonUIElements in the ButtonsRight collection. 
                var buttonElements = new List<EditorButtonUIElement>();
                int maxButtonWidth = 0;
                foreach (var button in buttonsRight)
                {
                    var buttonElement = editorWithTextUIElement.GetDescendant(typeof(EditorButtonUIElement), button) as EditorButtonUIElement;
                    if (null != buttonElement)
                    {
                        buttonElements.Add(buttonElement);
                        maxButtonWidth = Math.Max(maxButtonWidth, buttonElement.Rect.Width);
                    }
                }

                // Now calculate the rect in which the buttons will be positioned. 
                Rectangle rectInsideBorders = editorWithTextUIElement.RectInsideBorders;
                Rectangle buttonAreaRect = rectInsideBorders;
                buttonAreaRect.Width = maxButtonWidth;
                buttonAreaRect.X = rectInsideBorders.Right - buttonAreaRect.Width;
                int buttonHeight = buttonAreaRect.Height / buttonCount;

                // Position each button element.
                int buttonY = buttonAreaRect.Y;
                for (int i = 0; i< buttonElements.Count; i++)
                {
                    // Start with the original rect of the button element. 
                    var buttonElement = buttonElements[i];
                    Rectangle buttonRect = buttonElement.Rect;

                    // Set the height and the Y/Top position.
                    buttonRect.Height = buttonHeight;
                    buttonRect.Y = buttonY;

                    // In theory, all of the buttons are the same width, but just in case one of them 
                    // is smaller, center the button element withint the buttonAreaRect.
                    // Alternately, you would just set the Width here so that all buttons are the 
                    // same width. 
                    DrawUtility.AdjustHAlign(HAlign.Center, ref buttonRect, buttonAreaRect);

                    // Increment buttonY so we know the Y/Top of the next button. 
                    buttonY += buttonHeight;

                    // Set the rect on the button element. 
                    buttonElement.Rect = buttonRect;
                }

                // Now that we have positione the button, we need to expand the text area to 
                // fill the extra space. This will either be an EditorWithTextDisplayTextUIElement
                // (when not in edit mode) or a TextBox (when in edit mode). 
                // This code is assuming that there are no ButtonsLeft. If there are, we would
                // have to figure out where the left buttons are in order to determine the Left of
                // the text area. 
                //
                Rectangle textRect = parent.RectInsideBorders;
                textRect.Width -= buttonAreaRect.Width + 4; // I used 4 here just to add some padding so the text doesn't run into the buttons. 

                // If we are not in edit mode, we expect there to be an EditorWithTextDisplayTextUIElement
                var editorWithTextDisplayTextUIElement = parent.GetDescendant(typeof(EditorWithTextDisplayTextUIElement));
                if (null != editorWithTextDisplayTextUIElement)
                    editorWithTextDisplayTextUIElement.Rect = textRect;

                // If we are in edit mode, there will be a TextBox control instead of an EditorWithTextDisplayTextUIElement.
                if (parent.Control.Controls.Count > 0)
                {
                    var textBox = parent.Control.Controls[0];
                    textBox.SetBounds
                        (
                        textRect.X,
                        textRect.Y,
                        textRect.Width,
                        textRect.Height
                        );
                }
            }
        }

        bool IUIElementCreationFilter.BeforeCreateChildElements(UIElement parent)
        {
            return false;
        }       
    }
}
